/*
  Copyright ©2013 The Regents of the University of California
  (Regents). All Rights Reserved. Permission to use, copy, modify, and
  distribute this software and its documentation for educational,
  research, and not-for-profit purposes, without fee and without a
  signed licensing agreement, is hereby granted, provided that the
  above copyright notice, this paragraph and the following two
  paragraphs appear in all copies, modifications, and
  distributions. Contact The Office of Technology Licensing, UC
  Berkeley, 2150 Shattuck Avenue, Suite 510, Berkeley, CA 94720-1620,
  (510) 643-7201, for commercial licensing opportunities.

  IN NO EVENT SHALL REGENTS BE LIABLE TO ANY PARTY FOR DIRECT,
  INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
  LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
  DOCUMENTATION, EVEN IF REGENTS HAS BEEN ADVISED OF THE POSSIBILITY
  OF SUCH DAMAGE.

  REGENTS SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING
  DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS
  IS". REGENTS HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
  UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

#include "display.hpp"
#include "displayphysics.hpp"
#include "displayreplay.hpp"
#include "displaytesting.hpp"
#include "misc.hpp"
#include "io.hpp"
#include "runphysics.hpp"
#include <cstdlib>
#include <omp.h>
using namespace std;

// implement this anywhere in any .cpp file
extern void debug (const vector<string> &args);

int main (int argc, char **argv) {
    omp_set_num_threads(4); // too slow if more threads
    struct Action {
        string name;
        void (*run) (const vector<string> &args);
    } actions[] = {
        {"tri2obj", tri2obj},
        {"test", display_testing},
        {"physics", display_physics},
        // {"obj2png", obj2png},
        // {"obj2cyl", obj2cyl},
        {"offline", run_physics},
        {"resume", display_resume},
        {"resumeoffline", resume_physics},
        {"replay", display_replay},
        {"merge", merge_meshes},
        {"split", split_meshes},
        // {"offset", offset_meshes},
        {"handles", save_handles},
        {"debug", debug}
    };
    int nactions = sizeof(actions)/sizeof(Action);
    string name = (argc <= 1) ? "" : argv[1];
    vector<string> args;
    for (int a = 2; a < argc; a++)
        args.push_back(argv[a]);
    for (int i = 0; i < nactions; i++) {
        if (name == actions[i].name) {
            actions[i].run(args);
            exit(EXIT_SUCCESS);
        }
    }
    cout << "Unknown or unspecified action " << name << endl;
    cout << "Options are: ";
    for (int i = 0; i < nactions; i++)
        cout << actions[i].name << " ";
    cout << endl;
    cout << "Options for resume/resumeoffline: resume[offline] <output_prefix> <starting_frame>" << name << endl;
    cout << "Options for offset: offset <input format> <offset amount> <output format>" << name << endl;
    cout << "Options for subdiv: offset <input format> <num subdivision levels> <output format>" << name << endl;
    exit(EXIT_FAILURE);
}
